/*
 * Comm.cpp
 *
 *  Created on: Apr 7, 2018
 *      Author: armin
 */

#include <Comm.h>
#include <Work.h>
#include <arpa/inet.h>
#include <bits/stdint-uintn.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <unistd.h>
#include <cstring>
#include <sstream>
#include <stdexcept>

namespace comm {

using namespace std;

Comm::Comm(string host, int port) : port(port), host(host), sock(-1) {}

Work Comm::getWork(void) {
  checkConn();

  const char cmd[] = "WRK\n";
  if (send(sock, cmd, strlen(cmd), 0) < 0) {
    throw runtime_error("Sending WRK failed");
  }

  char buf[1024];
  ssize_t wrk_len;
  if ((wrk_len = recv(sock, buf, 1024, 0)) < 0) {
    throw runtime_error("Receiving of work failed");
  }

  buf[wrk_len] = '\0';

  if (!strcmp(buf, "NIL\n")) return (Work());

  string ws = string(buf, wrk_len - 1);
  Work wrk(ws);

  return (wrk);
}

void Comm::sendResult(Work work) {
  checkConn();

  const char cmd[] = "RES\n";
  if (send(sock, cmd, strlen(cmd), 0) < 0) {
    throw runtime_error("Sending RES failed");
  }

  string res = work.from.toHex() + " " + work.to.toHex() + " " + "\n";
  if (send(sock, res.c_str(), res.size(), 0) < 0) {
    throw runtime_error("Sending work result failed");
  }

  char buf[1024];
  ssize_t ack_len;
  if ((ack_len = recv(sock, buf, 1024, 0)) < 0) {
    throw runtime_error("Receiving ACK failed");
  }

  if (work.key.isEmpty()) {
    if (send(sock, "NIL\n", 4, 0) < 0) {
      throw runtime_error("Sending NIL ailed");
    }
  } else {
    if (send(sock, "SUC\n", 4, 0) < 0) {
      throw runtime_error("Sending SUC failed");
    }

    if ((ack_len = recv(sock, buf, 1024, 0)) < 0) {
      throw runtime_error("Receiving ACK failed");
    }

    string key = work.key.toHex() + "\n";
    if (send(sock, key.c_str(), key.length(), 0) < 0) {
      throw runtime_error("Sending key failed");
    }

    if ((ack_len = recv(sock, buf, 1024, 0)) < 0) {
      throw runtime_error("Receiving ACK failed");
    }
  }
}

void Comm::checkConn(void) {
  if (sock < 0) {
    if (!conn()) {
      ostringstream o;
      o << "Connection to " << host << ":" << port << " failed";
      throw runtime_error(o.str());
    }
  }
}

/**
 * Connects socket to server
 *
 * @return true if connected successfully, false otherwise
 */
bool Comm::conn() {
  if (sock > -1) {
    return (true);
  }

  sock = socket(AF_INET, SOCK_STREAM, 0);

  if (sock < 0) {
    return (false);
  }

  sockaddr_in serv_addr;

  serv_addr.sin_family = AF_INET;
  serv_addr.sin_port = htons((uint16_t)port);

  if (inet_pton(AF_INET, host.c_str(), &serv_addr.sin_addr) <= 0) {
    return (false);
  }

  if (connect(sock, reinterpret_cast<sockaddr*>(&serv_addr),
              sizeof(serv_addr)) < 0) {
    close(sock);
    return (false);
  }

  return (true);
}

Comm::~Comm() {
  shutdown(sock, SHUT_RDWR);
  close(sock);
  sock = -1;
}

}  // namespace comm
