package org.btcollider.cnc.dto;

import java.util.BitSet;
import org.btcollider.cnc.CnC;

public class KeyRange {
  private Long start;
  private Long end;

  public KeyRange(BitSet start, BitSet end) {
    assert start.length() <= CnC.MAX_BITS && end.length() <= CnC.MAX_BITS;

    this.start = start.toLongArray()[0];
    this.end = end.toLongArray()[0];
  }

  public KeyRange(Long start, Long end) {
    this.start = start;
    this.end = end;
  }

  public Long getStart() {
    return start;
  }

  public Long getEnd() {
    return end;
  }

  public Long getTotal() {
    // asserted that <= MAX_BITS, first long contains all significant bits
    Long total = getEnd() - getStart() + 1; // +1 bc start is included too

    return total;
  }

  @Override
  public String toString() {
    return "KeyRange [start=" + start + ", end=" + end + ", total=" + getTotal() + "]";
  }

  @Override
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((end == null) ? 0 : end.hashCode());
    result = prime * result + ((start == null) ? 0 : start.hashCode());
    return result;
  }

  @Override
  public boolean equals(Object obj) {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    KeyRange other = (KeyRange) obj;
    if (end == null) {
      if (other.end != null)
        return false;
    } else if (!end.equals(other.end))
      return false;
    if (start == null) {
      if (other.start != null)
        return false;
    } else if (!start.equals(other.start))
      return false;
    return true;
  }
}
