package org.btcollider.cnc.keysrv;

import java.util.concurrent.TimeUnit;
import org.btcollider.cnc.keysrv.impl.keytree.KeyTreeServer;
import org.btcollider.cnc.keysrv.impl.multistage.MultiStagedServer;

public class KSFactory {

  public static KeyServer build(int index, int depth, long maxWorkSpan) {
    return build(index, depth, maxWorkSpan, TimeUnit.MILLISECONDS);
  }

  public static KeyServer build(int index, int depth, long maxWorkSpan, TimeUnit timeUnit) {
    return build(index, depth, maxWorkSpan, timeUnit, 0);
  }

  public static KeyServer build(int index, int depth, long maxWorkSpan, TimeUnit timeUnit,
      float pruneFraction) {
    assert (pruneFraction >= 0.0 && pruneFraction <= 1.0);

    long maxKey = ((long) 1 << index) | (((long) 1 << index) - 1);
    long pruneKey = (long) Math.floor(maxKey * pruneFraction);

    return build(index, depth, maxWorkSpan, timeUnit, pruneKey);
  }

  public static KeyServer build(int index, int depth, long maxWorkSpan, TimeUnit timeUnit,
      long pruneKey) {
    if (depth == 25) {
      return new KeyTreeServer(index, depth, timeUnit.toMillis(maxWorkSpan), pruneKey);
    } else {
      return new MultiStagedServer(index, depth, timeUnit.toMillis(maxWorkSpan), pruneKey);
    }

  }
}
