package org.btcollider.cnc.keysrv;

import org.btcollider.cnc.dto.KeyRange;

public interface KeyServer {
  /**
   * Gets a free (= not setInWork and not setSearched) KeyRange from the pool of possible key ranges
   *
   * @return A free KeyRange or null if none such exists
   */
  KeyRange getRange();

  /**
   * Sets a KeyRange as being currently worked on.
   *
   * As long as a KeyRange is not setSearched, there is no guarantee that an inWork KeyRange will be
   * finished in any specific time frame. The KeyServer has to clean up dangling inWorks in good
   * faith. Unexpected setSearched of already cleaned-up inWorks must not be of any harm.
   *
   * @param keyRange The KeyRange to be set as being worked on, not null
   */
  void setInWork(KeyRange keyRange);

  /**
   * Sets a KeyRange as being searched, finalizing this range A searched KeyRange need not be
   * processed any more
   *
   * @param keyRange The KeyRange to be set as searched, not null
   */
  void setSearched(KeyRange keyRange);

}
